@@ -16,8 +16,6 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/channel/connected_channel.h"
 
 #include <inttypes.h>
@@ -28,6 +26,7 @@
 #include <type_traits>
 #include <utility>
 
+#include "absl/log/check.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/types/optional.h"
@@ -36,6 +35,7 @@
 #include <grpc/status.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/lib/channel/call_finalization.h"
 #include "src/core/lib/channel/channel_args.h"
@@ -225,7 +225,7 @@ static void connected_channel_destroy_call_elem(
 static grpc_error_handle connected_channel_init_channel_elem(
     grpc_channel_element* elem, grpc_channel_element_args* args) {
   channel_data* cd = static_cast<channel_data*>(elem->channel_data);
-  GPR_ASSERT(args->is_last);
+  CHECK(args->is_last);
   cd->transport = args->channel_args.GetObject<grpc_core::Transport>();
   return absl::OkStatus();
 }
@@ -364,7 +364,7 @@ class ConnectedChannelStream : public Orphanable {
   grpc_stream_refcount stream_refcount_;
   StreamPtr stream_;
   Arena* arena_ = GetContext<Arena>();
-  Party* const party_ = static_cast<Party*>(Activity::current());
+  Party* const party_ = GetContext<Party>();
   ExternallyObservableLatch<void> finished_;
 };
 
@@ -383,17 +383,18 @@ auto ConnectedChannelStream::RecvMessages(
               gpr_log(GPR_INFO,
                       "%s[connected] RecvMessage: received payload of %" PRIdPTR
                       " bytes",
-                      Activity::current()->DebugTag().c_str(),
+                      GetContext<Activity>()->DebugTag().c_str(),
                       pending_message->payload()->Length());
             }
             return Map(incoming_messages.Push(std::move(pending_message)),
                        [](bool ok) -> LoopCtl<absl::Status> {
                          if (!ok) {
                            if (grpc_call_trace.enabled()) {
-                             gpr_log(GPR_INFO,
-                                     "%s[connected] RecvMessage: failed to "
-                                     "push message towards the application",
-                                     Activity::current()->DebugTag().c_str());
+                             gpr_log(
+                                 GPR_INFO,
+                                 "%s[connected] RecvMessage: failed to "
+                                 "push message towards the application",
+                                 GetContext<Activity>()->DebugTag().c_str());
                            }
                            return absl::OkStatus();
                          }
@@ -406,7 +407,7 @@ auto ConnectedChannelStream::RecvMessages(
               gpr_log(GPR_INFO,
                       "%s[connected] RecvMessage: reached end of stream with "
                       "status:%s",
-                      Activity::current()->DebugTag().c_str(),
+                      GetContext<Activity>()->DebugTag().c_str(),
                       status.status().ToString().c_str());
             }
             if (cancel_on_error && !status.ok()) {
@@ -444,7 +445,7 @@ ArenaPromise<ServerMetadataHandle> MakeClientCallPromise(Transport* transport,
   transport->filter_stack_transport()->InitStream(stream->stream(),
                                                   stream->stream_refcount(),
                                                   nullptr, GetContext<Arena>());
-  auto* party = static_cast<Party*>(Activity::current());
+  auto* party = GetContext<Party>();
   party->Spawn("set_polling_entity", call_args.polling_entity->Wait(),
                [transport, stream = stream->InternalRef()](
                    grpc_polling_entity polling_entity) {
@@ -463,8 +464,7 @@ ArenaPromise<ServerMetadataHandle> MakeClientCallPromise(Transport* transport,
       [](absl::Status) {});
   // Start a promise to receive server initial metadata and then forward it up
   // through the receiving pipe.
-  auto server_initial_metadata =
-      GetContext<Arena>()->MakePooled<ServerMetadata>(GetContext<Arena>());
+  auto server_initial_metadata = Arena::MakePooled<ServerMetadata>();
   party->Spawn(
       "recv_initial_metadata",
       TrySeq(GetContext<BatchBuilder>()->ReceiveServerInitialMetadata(
@@ -474,7 +474,7 @@ ArenaPromise<ServerMetadataHandle> MakeClientCallPromise(Transport* transport,
                if (grpc_call_trace.enabled()) {
                  gpr_log(GPR_DEBUG,
                          "%s[connected] Publish client initial metadata: %s",
-                         Activity::current()->DebugTag().c_str(),
+                         GetContext<Activity>()->DebugTag().c_str(),
                          server_initial_metadata->DebugString().c_str());
                }
                return Map(pipe->Push(std::move(server_initial_metadata)),
@@ -501,28 +501,25 @@ ArenaPromise<ServerMetadataHandle> MakeClientCallPromise(Transport* transport,
   // Create a promise that will receive server trailing metadata.
   // If this fails, we massage the error into metadata that we can report
   // upwards.
-  auto server_trailing_metadata =
-      GetContext<Arena>()->MakePooled<ServerMetadata>(GetContext<Arena>());
-  auto recv_trailing_metadata =
-      Map(GetContext<BatchBuilder>()->ReceiveServerTrailingMetadata(
-              stream->batch_target()),
-          [](absl::StatusOr<ServerMetadataHandle> status) mutable {
-            if (!status.ok()) {
-              auto server_trailing_metadata =
-                  GetContext<Arena>()->MakePooled<ServerMetadata>(
-                      GetContext<Arena>());
-              grpc_status_code status_code = GRPC_STATUS_UNKNOWN;
-              std::string message;
-              grpc_error_get_status(status.status(), Timestamp::InfFuture(),
-                                    &status_code, &message, nullptr, nullptr);
-              server_trailing_metadata->Set(GrpcStatusMetadata(), status_code);
-              server_trailing_metadata->Set(GrpcMessageMetadata(),
-                                            Slice::FromCopiedString(message));
-              return server_trailing_metadata;
-            } else {
-              return std::move(*status);
-            }
-          });
+  auto server_trailing_metadata = Arena::MakePooled<ServerMetadata>();
+  auto recv_trailing_metadata = Map(
+      GetContext<BatchBuilder>()->ReceiveServerTrailingMetadata(
+          stream->batch_target()),
+      [](absl::StatusOr<ServerMetadataHandle> status) mutable {
+        if (!status.ok()) {
+          auto server_trailing_metadata = Arena::MakePooled<ServerMetadata>();
+          grpc_status_code status_code = GRPC_STATUS_UNKNOWN;
+          std::string message;
+          grpc_error_get_status(status.status(), Timestamp::InfFuture(),
+                                &status_code, &message, nullptr, nullptr);
+          server_trailing_metadata->Set(GrpcStatusMetadata(), status_code);
+          server_trailing_metadata->Set(GrpcMessageMetadata(),
+                                        Slice::FromCopiedString(message));
+          return server_trailing_metadata;
+        } else {
+          return std::move(*status);
+        }
+      });
   // Finally the main call promise.
   // Concurrently: send initial metadata and receive messages, until BOTH
   // complete (or one fails).
@@ -581,7 +578,7 @@ ArenaPromise<ServerMetadataHandle> MakeServerCallPromise(
       stream->stream(), stream->stream_refcount(),
       GetContext<CallContext>()->server_call_context()->server_stream_data(),
       GetContext<Arena>());
-  auto* party = static_cast<Party*>(Activity::current());
+  auto* party = GetContext<Party>();
 
   // Arifacts we need for the lifetime of the call.
   struct CallData {
@@ -747,7 +744,7 @@ ArenaPromise<ServerMetadataHandle> MakeServerCallPromise(
                       : nullptr;
         if (md != nullptr) {
           call_data->sent_initial_metadata = true;
-          auto* party = static_cast<Party*>(Activity::current());
+          auto* party = GetContext<Party>();
           party->Spawn("connected/send_initial_metadata",
                        GetContext<BatchBuilder>()->SendServerInitialMetadata(
                            stream->batch_target(), std::move(md)),
@@ -777,7 +774,7 @@ ArenaPromise<ServerMetadataHandle> MakeServerCallPromise(
               gpr_log(
                   GPR_DEBUG,
                   "%s[connected] Got trailing metadata; status=%s metadata=%s",
-                  Activity::current()->DebugTag().c_str(),
+                  GetContext<Activity>()->DebugTag().c_str(),
                   status.status().ToString().c_str(),
                   status.ok() ? (*status)->DebugString().c_str() : "<none>");
             }
@@ -785,9 +782,7 @@ ArenaPromise<ServerMetadataHandle> MakeServerCallPromise(
             if (status.ok()) {
               trailing_metadata = std::move(*status);
             } else {
-              trailing_metadata =
-                  GetContext<Arena>()->MakePooled<ClientMetadata>(
-                      GetContext<Arena>());
+              trailing_metadata = Arena::MakePooled<ClientMetadata>();
               grpc_status_code status_code = GRPC_STATUS_UNKNOWN;
               std::string message;
               grpc_error_get_status(status.status(), Timestamp::InfFuture(),
@@ -890,19 +885,7 @@ ArenaPromise<ServerMetadataHandle> MakeClientTransportCallPromise(
     Transport* transport, CallArgs call_args, NextPromiseFactory) {
   auto spine = GetContext<CallContext>()->MakeCallSpine(std::move(call_args));
   transport->client_transport()->StartCall(CallHandler{spine});
-  return Map(spine->server_trailing_metadata().receiver.Next(),
-             [](NextResult<ServerMetadataHandle> r) {
-               if (r.has_value()) {
-                 auto md = std::move(r.value());
-                 md->Set(GrpcStatusFromWire(), true);
-                 return md;
-               }
-               auto m = GetContext<Arena>()->MakePooled<ServerMetadata>(
-                   GetContext<Arena>());
-               m->Set(GrpcStatusMetadata(), GRPC_STATUS_CANCELLED);
-               m->Set(GrpcCallWasCancelled(), true);
-               return m;
-             });
+  return spine->PullServerTrailingMetadata();
 }
 
 const grpc_channel_filter kClientPromiseBasedTransportFilter =
