@@ -30,19 +30,20 @@
 #include <grpc/grpc.h>
 #include <grpc/support/log.h>
 
-#include "src/core/ext/xds/xds_bootstrap.h"
-#include "src/core/ext/xds/xds_bootstrap_grpc.h"
-#include "src/core/ext/xds/xds_client.h"
-#include "src/core/ext/xds/xds_cluster.h"
-#include "src/core/ext/xds/xds_endpoint.h"
-#include "src/core/ext/xds/xds_listener.h"
-#include "src/core/ext/xds/xds_route_config.h"
 #include "src/core/lib/event_engine/default_event_engine.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
+#include "src/core/xds/grpc/xds_bootstrap_grpc.h"
+#include "src/core/xds/grpc/xds_cluster.h"
+#include "src/core/xds/grpc/xds_endpoint.h"
+#include "src/core/xds/grpc/xds_listener.h"
+#include "src/core/xds/grpc/xds_route_config.h"
+#include "src/core/xds/xds_client/xds_bootstrap.h"
+#include "src/core/xds/xds_client/xds_client.h"
 #include "src/libfuzzer/libfuzzer_macro.h"
 #include "src/proto/grpc/testing/xds/v3/discovery.pb.h"
 #include "test/core/xds/xds_client_fuzzer.pb.h"
+#include "test/core/xds/xds_client_test_peer.h"
 #include "test/core/xds/xds_transport_fake.h"
 
 namespace grpc_core {
@@ -64,8 +65,8 @@ class Fuzzer {
     transport_factory_ = transport_factory.get();
     xds_client_ = MakeRefCounted<XdsClient>(
         std::move(*bootstrap), std::move(transport_factory),
-        grpc_event_engine::experimental::GetDefaultEventEngine(), "foo agent",
-        "foo version");
+        grpc_event_engine::experimental::GetDefaultEventEngine(),
+        /*metrics_reporter=*/nullptr, "foo agent", "foo version");
   }
 
   void Act(const xds_client_fuzzer::Action& action) {
@@ -113,7 +114,29 @@ class Fuzzer {
         }
         break;
       case xds_client_fuzzer::Action::kDumpCsdsData:
-        xds_client_->DumpClientConfigBinary();
+        testing::XdsClientTestPeer(xds_client_.get()).TestDumpClientConfig();
+        break;
+      case xds_client_fuzzer::Action::kReportResourceCounts:
+        testing::XdsClientTestPeer(xds_client_.get())
+            .TestReportResourceCounts(
+                [](const testing::XdsClientTestPeer::ResourceCountLabels&
+                       labels,
+                   uint64_t count) {
+                  gpr_log(GPR_INFO,
+                          "xds_authority=\"%s\", resource_type=\"%s\", "
+                          "cache_state=\"%s\" count=%" PRIu64,
+                          std::string(labels.xds_authority).c_str(),
+                          std::string(labels.resource_type).c_str(),
+                          std::string(labels.cache_state).c_str(), count);
+                });
+        break;
+      case xds_client_fuzzer::Action::kReportServerConnections:
+        testing::XdsClientTestPeer(xds_client_.get())
+            .TestReportServerConnections(
+                [](absl::string_view xds_server, bool connected) {
+                  gpr_log(GPR_INFO, "xds_server=\"%s\" connected=%d",
+                          std::string(xds_server).c_str(), connected);
+                });
         break;
       case xds_client_fuzzer::Action::kTriggerConnectionFailure:
         TriggerConnectionFailure(
@@ -216,13 +239,15 @@ class Fuzzer {
   const XdsBootstrap::XdsServer* GetServer(const std::string& authority) {
     const GrpcXdsBootstrap& bootstrap =
         static_cast<const GrpcXdsBootstrap&>(xds_client_->bootstrap());
-    if (authority.empty()) return &bootstrap.server();
+    if (authority.empty()) return bootstrap.servers().front();
     const auto* authority_entry =
         static_cast<const GrpcXdsBootstrap::GrpcAuthority*>(
             bootstrap.LookupAuthority(authority));
     if (authority_entry == nullptr) return nullptr;
-    if (authority_entry->server() != nullptr) return authority_entry->server();
-    return &bootstrap.server();
+    if (!authority_entry->servers().empty()) {
+      return authority_entry->servers().front();
+    }
+    return bootstrap.servers().front();
   }
 
   void TriggerConnectionFailure(const std::string& authority,
