@@ -27,11 +27,10 @@
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/channel/call_tracer.h"
 #include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/experiments/experiments.h"
 #include "src/core/lib/surface/call.h"
 #include "src/proto/grpc/testing/xds/v3/orca_load_report.pb.h"
-#include "test/core/util/fake_stats_plugin.h"
-#include "test/core/util/scoped_env_var.h"
+#include "test/core/test_util/fake_stats_plugin.h"
+#include "test/core/test_util/scoped_env_var.h"
 #include "test/cpp/end2end/connection_attempt_injector.h"
 #include "test/cpp/end2end/xds/xds_end2end_test_lib.h"
 
@@ -45,8 +44,8 @@ using ::envoy::config::core::v3::HealthStatus;
 using ::envoy::type::v3::FractionalPercent;
 
 using ClientStats = LrsServiceImpl::ClientStats;
-using OptionalLabelComponent =
-    grpc_core::ClientCallTracer::CallAttemptTracer::OptionalLabelComponent;
+using OptionalLabelKey =
+    grpc_core::ClientCallTracer::CallAttemptTracer::OptionalLabelKey;
 
 constexpr char kLbDropType[] = "lb";
 constexpr char kThrottleDropType[] = "throttle";
@@ -309,12 +308,13 @@ TEST_P(CdsTest, ClusterChangeAfterAdsCallFails) {
   WaitForBackend(DEBUG_LOCATION, 1);
 }
 
-TEST_P(CdsTest, VerifyCsmServiceLabelsParsing) {
+TEST_P(CdsTest, MetricLabels) {
   // Injects a fake client call tracer factory. Try keep this at top.
   grpc_core::FakeClientCallTracerFactory fake_client_call_tracer_factory;
-  CreateAndStartBackends(1);
+  CreateAndStartBackends(2);
   // Populates EDS resources.
-  EdsResourceArgs args({{"locality0", CreateEndpointsForBackends()}});
+  EdsResourceArgs args({{"locality0", CreateEndpointsForBackends(0, 1)},
+                        {"locality1", CreateEndpointsForBackends(1, 2)}});
   balancer_->ads_service()->SetEdsResource(BuildEdsResource(args));
   // Populates service labels to CDS resources.
   auto cluster = default_cluster_;
@@ -328,18 +328,33 @@ TEST_P(CdsTest, VerifyCsmServiceLabelsParsing) {
   channel_args.SetPointer(GRPC_ARG_INJECT_FAKE_CLIENT_CALL_TRACER_FACTORY,
                           &fake_client_call_tracer_factory);
   ResetStub(/*failover_timeout_ms=*/0, &channel_args);
-  // Sends an RPC and verifies that the service labels are recorded in the fake
-  // client call tracer.
-  CheckRpcSendOk(DEBUG_LOCATION);
-  EXPECT_THAT(fake_client_call_tracer_factory.GetLastFakeClientCallTracer()
-                  ->GetLastCallAttemptTracer()
-                  ->GetOptionalLabels(),
-              ::testing::ElementsAre(::testing::Pair(
-                  OptionalLabelComponent::kXdsServiceLabels,
-                  ::testing::Pointee(::testing::ElementsAre(
-                      ::testing::Pair("service_name", "myservice"),
-                      ::testing::Pair("service_namespace", "mynamespace"))))));
-  balancer_->Shutdown();
+  // Send an RPC to backend 0.
+  WaitForBackend(DEBUG_LOCATION, 0);
+  // Verify that the optional labels are recorded in the call tracer.
+  EXPECT_THAT(
+      fake_client_call_tracer_factory.GetLastFakeClientCallTracer()
+          ->GetLastCallAttemptTracer()
+          ->GetOptionalLabels(),
+      ::testing::ElementsAre(
+          ::testing::Pair(OptionalLabelKey::kXdsServiceName, "myservice"),
+          ::testing::Pair(OptionalLabelKey::kXdsServiceNamespace,
+                          "mynamespace"),
+          ::testing::Pair(OptionalLabelKey::kLocality,
+                          LocalityNameString("locality0"))));
+  // Send an RPC to backend 1.
+  WaitForBackend(DEBUG_LOCATION, 1);
+  // Verify that the optional labels are recorded in the call
+  // tracer.
+  EXPECT_THAT(
+      fake_client_call_tracer_factory.GetLastFakeClientCallTracer()
+          ->GetLastCallAttemptTracer()
+          ->GetOptionalLabels(),
+      ::testing::ElementsAre(
+          ::testing::Pair(OptionalLabelKey::kXdsServiceName, "myservice"),
+          ::testing::Pair(OptionalLabelKey::kXdsServiceNamespace,
+                          "mynamespace"),
+          ::testing::Pair(OptionalLabelKey::kLocality,
+                          LocalityNameString("locality1"))));
 }
 
 //
@@ -381,7 +396,7 @@ TEST_P(CdsDeletionTest, ClusterDeleted) {
 
 // Tests that we ignore Cluster deletions if configured to do so.
 TEST_P(CdsDeletionTest, ClusterDeletionIgnored) {
-  InitClient(XdsBootstrapBuilder().SetIgnoreResourceDeletion());
+  InitClient(MakeBootstrapBuilder().SetIgnoreResourceDeletion());
   CreateAndStartBackends(2);
   // Bring up client pointing to backend 0 and wait for it to connect.
   EdsResourceArgs args({{"locality0", CreateEndpointsForBackends(0, 1)}});
@@ -452,7 +467,6 @@ TEST_P(EdsTest, Vanilla) {
 }
 
 TEST_P(EdsTest, MultipleAddressesPerEndpoint) {
-  if (!grpc_core::IsRoundRobinDelegateToPickFirstEnabled()) return;
   grpc_core::testing::ScopedExperimentalEnvVar env(
       "GRPC_EXPERIMENTAL_XDS_DUALSTACK_ENDPOINTS");
   const size_t kNumRpcsPerAddress = 10;
