@@ -34,14 +34,14 @@
 #include <grpcpp/security/tls_certificate_provider.h>
 
 #include "src/core/ext/filters/http/server/http_server_filter.h"
-#include "src/core/ext/xds/xds_channel_args.h"
-#include "src/core/ext/xds/xds_client_grpc.h"
 #include "src/core/lib/gpr/tmpfile.h"
 #include "src/core/lib/gprpp/env.h"
-#include "src/core/lib/surface/server.h"
+#include "src/core/server/server.h"
+#include "src/core/xds/grpc/xds_client_grpc.h"
+#include "src/core/xds/xds_client/xds_channel_args.h"
 #include "src/cpp/client/secure_credentials.h"
 #include "src/proto/grpc/testing/xds/v3/router.grpc.pb.h"
-#include "test/core/util/resolve_localhost_ip46.h"
+#include "test/core/test_util/resolve_localhost_ip46.h"
 
 namespace grpc {
 namespace testing {
@@ -60,7 +60,7 @@ using ::envoy::extensions::filters::network::http_connection_manager::v3::
 
 std::string XdsBootstrapBuilder::Build() {
   std::vector<std::string> fields;
-  fields.push_back(MakeXdsServersText(top_server_));
+  fields.push_back(MakeXdsServersText(servers_));
   if (!client_default_listener_resource_name_template_.empty()) {
     fields.push_back(
         absl::StrCat("  \"client_default_listener_resource_name_template\": \"",
@@ -78,9 +78,8 @@ std::string XdsBootstrapBuilder::Build() {
 }
 
 std::string XdsBootstrapBuilder::MakeXdsServersText(
-    absl::string_view server_uri) {
+    absl::Span<const std::string> server_uris) {
   constexpr char kXdsServerTemplate[] =
-      "      \"xds_servers\": [\n"
       "        {\n"
       "          \"server_uri\": \"<SERVER_URI>\",\n"
       "          \"channel_creds\": [\n"
@@ -89,17 +88,21 @@ std::string XdsBootstrapBuilder::MakeXdsServersText(
       "            }\n"
       "          ],\n"
       "          \"server_features\": [<SERVER_FEATURES>]\n"
-      "        }\n"
-      "      ]";
+      "        }";
   std::vector<std::string> server_features;
   if (ignore_resource_deletion_) {
     server_features.push_back("\"ignore_resource_deletion\"");
   }
-  return absl::StrReplaceAll(
-      kXdsServerTemplate,
-      {{"<SERVER_URI>", server_uri},
-       {"<SERVER_CREDS_TYPE>", xds_channel_creds_type_},
-       {"<SERVER_FEATURES>", absl::StrJoin(server_features, ", ")}});
+  std::vector<std::string> servers;
+  for (absl::string_view server_uri : server_uris) {
+    servers.emplace_back(absl::StrReplaceAll(
+        kXdsServerTemplate,
+        {{"<SERVER_URI>", server_uri},
+         {"<SERVER_CREDS_TYPE>", xds_channel_creds_type_},
+         {"<SERVER_FEATURES>", absl::StrJoin(server_features, ", ")}}));
+  }
+  return absl::StrCat("      \"xds_servers\": [\n",
+                      absl::StrJoin(servers, ",\n"), "\n      ]");
 }
 
 std::string XdsBootstrapBuilder::MakeNodeText() {
@@ -148,7 +151,7 @@ std::string XdsBootstrapBuilder::MakeAuthorityText() {
     const std::string& name = p.first;
     const AuthorityInfo& authority_info = p.second;
     std::vector<std::string> fields = {
-        MakeXdsServersText(authority_info.server)};
+        MakeXdsServersText(authority_info.servers)};
     if (!authority_info.client_listener_resource_name_template.empty()) {
       fields.push_back(absl::StrCat(
           "\"client_listener_resource_name_template\": \"",
@@ -317,6 +320,12 @@ void XdsResourceUtils::SetRouteConfiguration(
   }
 }
 
+std::string XdsResourceUtils::LocalityNameString(absl::string_view sub_zone) {
+  return absl::StrFormat("{region=\"%s\", zone=\"%s\", sub_zone=\"%s\"}",
+                         kDefaultLocalityRegion, kDefaultLocalityZone,
+                         sub_zone);
+}
+
 ClusterLoadAssignment XdsResourceUtils::BuildEdsResource(
     const EdsResourceArgs& args, absl::string_view eds_service_name) {
   ClusterLoadAssignment assignment;
