@@ -130,6 +130,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.57.0", ReleaseInfo()),
             ("v1.58.0", ReleaseInfo()),
             ("v1.60.0", ReleaseInfo()),
+            ("v1.61.0", ReleaseInfo()),
+            ("v1.62.0", ReleaseInfo()),
+            ("v1.63.0", ReleaseInfo()),
         ]
     ),
     "go": OrderedDict(
@@ -294,6 +297,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.58.3", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.59.0", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.60.1", ReleaseInfo(runtimes=["go1.19"])),
+            ("v1.61.2", ReleaseInfo(runtimes=["go1.19"])),
+            ("v1.62.2", ReleaseInfo(runtimes=["go1.19"])),
+            ("v1.63.2", ReleaseInfo(runtimes=["go1.19"])),
         ]
     ),
     "java": OrderedDict(
@@ -435,6 +441,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.59.1", ReleaseInfo()),
             ("v1.60.1", ReleaseInfo()),
             ("v1.61.0", ReleaseInfo()),
+            ("v1.63.0", ReleaseInfo()),
         ]
     ),
     "python": OrderedDict(
@@ -763,6 +770,24 @@ LANG_RELEASE_MATRIX = {
                     runtimes=["python"], testcases_file="python__master"
                 ),
             ),
+            (
+                "v1.61.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
+            (
+                "v1.62.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
+            (
+                "v1.63.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
         ]
     ),
     "node": OrderedDict(
@@ -857,6 +882,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.57.0", ReleaseInfo()),
             ("v1.58.0", ReleaseInfo()),
             ("v1.60.0", ReleaseInfo()),
+            ("v1.61.0", ReleaseInfo()),
+            ("v1.62.0", ReleaseInfo()),
+            ("v1.63.0", ReleaseInfo()),
         ]
     ),
     "php": OrderedDict(
@@ -915,6 +943,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.57.0", ReleaseInfo()),
             ("v1.58.0", ReleaseInfo()),
             ("v1.60.0", ReleaseInfo()),
+            ("v1.61.0", ReleaseInfo()),
+            ("v1.62.0", ReleaseInfo()),
+            ("v1.63.0", ReleaseInfo()),
         ]
     ),
     "csharp": OrderedDict(
